@@ -22,14 +22,15 @@
 #include "config.h"
 
 #include "seahorse-service.h"
-#include "seahorse-key.h"
+#include "seahorse-object.h"
 #include "seahorse-gconf.h"
 #include "seahorse-util.h"
 #include "seahorse-libdialogs.h"
 
-#include "pgp/seahorse-pgp-dialogs.h"
-#include "pgp/seahorse-pgp-key.h"
-#include "pgp/seahorse-pgp-operation.h"
+#include "pgp/seahorse-gpgme.h"
+#include "pgp/seahorse-gpgme-dialogs.h"
+#include "pgp/seahorse-gpgme-key.h"
+#include "pgp/seahorse-gpgme-operation.h"
 
 #include <string.h>
 
@@ -45,10 +46,22 @@ G_DEFINE_TYPE (SeahorseServiceCrypto, seahorse_service_crypto, G_TYPE_OBJECT);
    to other processes in the case of notifications */
 #define ICON_PREFIX     DATA_DIR "/pixmaps/seahorse/48x48/"
 
+/**
+ * SECTION:seahorse-service-crypto
+ * @short_description: Seahorse service crypto DBus methods. The other
+ * DBus methods can be found in other files
+ *
+ **/
+
 /* -----------------------------------------------------------------------------
  * PUBLIC METHODS
  */
- 
+
+/**
+*
+* Creates and returns a new crypto service
+* (SEAHORSE_TYPE_SERVICE_CRYPTO)
+*/
 SeahorseServiceCrypto* 
 seahorse_service_crypto_new ()
 {
@@ -60,8 +73,19 @@ seahorse_service_crypto_new ()
  */
 
 /* Returns result in result. Frees pop and cryptdata */
+/**
+* pop: a seahorse operation
+* gstarterr: the gpgme error that could have occured earlier
+* cryptdata: gpgme cryptdata
+* result: the result of the gpgme operation (out)
+* error: will be set on error
+*
+* Finishes the gpgme processing and returns the result
+*
+* returns: TRUE on successful operation, FALSE else
+*/
 static gboolean
-process_crypto_result (SeahorsePGPOperation *pop, gpgme_error_t gstarterr, 
+process_crypto_result (SeahorseGpgmeOperation *pop, gpgme_error_t gstarterr, 
                        gpgme_data_t cryptdata, gchar **result, GError **error)
 {
     size_t len;
@@ -79,7 +103,12 @@ process_crypto_result (SeahorsePGPOperation *pop, gpgme_error_t gstarterr,
     /* Wait for it to finish (can run main loop stuff) */
     seahorse_operation_wait (SEAHORSE_OPERATION (pop));
     
-    if (seahorse_operation_is_successful (SEAHORSE_OPERATION (pop))) {
+    if (seahorse_operation_is_cancelled (SEAHORSE_OPERATION (pop))) {
+	    
+	    g_set_error (error, SEAHORSE_DBUS_CANCELLED, 0, "%s", "");
+	    return FALSE;
+    
+    } else if (seahorse_operation_is_successful (SEAHORSE_OPERATION (pop))) {
         
         data = gpgme_data_release_and_get_mem (cryptdata, &len);
         *result = g_strndup (data, len);
@@ -99,6 +128,12 @@ process_crypto_result (SeahorsePGPOperation *pop, gpgme_error_t gstarterr,
     }
 }
 
+/**
+* keys: (GList)
+*
+* Creates a gpgme_key_t array out of the keylist.
+*
+*/
 static gpgme_key_t* 
 keylist_to_keys (GList *keys)
 {
@@ -108,23 +143,48 @@ keylist_to_keys (GList *keys)
 	recips = g_new0 (gpgme_key_t, g_list_length (keys) + 1);
 
 	for (i = 0; keys != NULL; keys = g_list_next (keys), i++) {
-		g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (keys->data), recips);
-		recips[i] = SEAHORSE_PGP_KEY (keys->data)->pubkey;
-		gpgmex_key_ref (recips[i]);
+		g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (keys->data), recips);
+		recips[i] = seahorse_gpgme_key_get_public (keys->data);
+		gpgme_key_ref (recips[i]);
 	}
 
 	return recips;
 }
 
+
+/**
+* keys: gpgme_key_t list of keys
+*
+* Frees the keys
+*
+*/
+static void
+free_keys (gpgme_key_t* keys)
+{
+	gpgme_key_t* k = keys;
+	if (!keys)
+		return;
+	while (*k)
+		gpgme_key_unref (*(k++));
+	g_free (keys);
+}
+
+/**
+* data: optional, will be added to the title, can be NULL
+* status: the gpgme status
+*
+* Basing on the status a notification is created an displayed
+*
+*/
 static void
 notify_signatures (const gchar* data, gpgme_verify_result_t status)
 {
 	const gchar *icon = NULL;
+	SeahorseObject *object;
 	gchar *title, *body;
 	gboolean sig = FALSE;
 	GSList *rawids;
 	GList *keys;
-	SeahorseKey *skey;
 
 	/* Discover the key in question */
 	rawids = g_slist_append (NULL, status->signatures->fpr);
@@ -132,7 +192,7 @@ notify_signatures (const gchar* data, gpgme_verify_result_t status)
 	g_slist_free (rawids);
 
 	g_return_if_fail (keys != NULL);
-	skey = SEAHORSE_KEY (keys->data);
+	object = SEAHORSE_OBJECT (keys->data);
 	g_list_free (keys);
 
 	/* Figure out what to display */
@@ -179,14 +239,14 @@ notify_signatures (const gchar* data, gpgme_verify_result_t status)
 		return;
 	default:
 		if (!GPG_IS_OK (status->signatures->status))
-			seahorse_pgp_handle_gpgme_error (status->signatures->status, 
-			                            _("Couldn't verify signature."));
+			seahorse_gpgme_handle_error (status->signatures->status, 
+			                             _("Couldn't verify signature."));
 		return;
 	};
 
 	if (sig) {
 		gchar *date = seahorse_util_get_display_date_string (status->signatures->timestamp);
-		gchar *id = seahorse_context_id_to_dbus (SCTX_APP (), seahorse_key_get_keyid (skey), 0);
+		gchar *id = seahorse_context_id_to_dbus (SCTX_APP (), seahorse_object_get_id (object));
 		body = g_markup_printf_escaped (body, id, date);
 		g_free (date);
 		g_free (id);
@@ -211,6 +271,21 @@ notify_signatures (const gchar* data, gpgme_verify_result_t status)
  * DBUS METHODS 
  */
 
+/**
+ * seahorse_service_crypto_encrypt_text:
+ * @crypto: the crypto service (#SeahorseServiceCrypto)
+ * @recipients: A list of recipients (keyids "openpgp:B8098FB063E2C811")
+ * @signer: optional
+ * @flags: 0, not used
+ * @cleartext: the text to encrypt
+ * @crypttext: the decrypted text (out)
+ * @error: an error (out)
+ *
+ * DBus: EncryptText
+ *
+ *
+ * Returns: TRUE on success
+ */
 gboolean
 seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto, 
                                       const char **recipients, const char *signer, 
@@ -218,7 +293,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
                                       char **crypttext, GError **error)
 {
     GList *recipkeys = NULL;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     SeahorseObject *signkey = NULL;
     SeahorseObject *skey;
     gpgme_key_t *recips;
@@ -233,15 +308,15 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     
     /* The signer */
     if (signer && signer[0]) {
-        signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer, NULL);
+        signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer);
         if (!signkey) {
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
                          _("Invalid or unrecognized signer: %s"), signer);
             return FALSE;
         }
         
-        if (!SEAHORSE_IS_PGP_KEY (signkey) || 
-            !(seahorse_object_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
+        if (!SEAHORSE_IS_GPGME_KEY (signkey) || 
+            !(seahorse_object_get_flags (signkey) & SEAHORSE_FLAG_CAN_SIGN)) {
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                          _("Key is not valid for signing: %s"), signer);
             return FALSE;
@@ -251,7 +326,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     /* The recipients */
     for( ; recipients[0]; recipients++)
     {
-        skey = seahorse_context_object_from_dbus (SCTX_APP (), recipients[0], NULL);
+        skey = seahorse_context_object_from_dbus (SCTX_APP (), recipients[0]);
         if (!skey) {
             g_list_free (recipkeys);
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
@@ -259,8 +334,8 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
             return FALSE;
         }
         
-        if (!SEAHORSE_IS_PGP_KEY (skey) ||
-            !(seahorse_object_get_flags (skey) & SKEY_FLAG_CAN_ENCRYPT)) {
+        if (!SEAHORSE_IS_GPGME_KEY (skey) ||
+            !(seahorse_object_get_flags (skey) & SEAHORSE_FLAG_CAN_ENCRYPT)) {
             g_list_free (recipkeys);
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                          _("Key is not a valid recipient for encryption: %s"), recipients[0]);
@@ -276,7 +351,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
         return FALSE;
     }
     
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data form text */
     gerr = gpgme_data_new_from_mem (&plain, cleartext, strlen (cleartext), FALSE);
@@ -301,7 +376,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     
     /* Do the encryption */
     if (signkey) {
-        gpgme_signers_add (pop->gctx, SEAHORSE_PGP_KEY (signkey)->seckey);
+        gpgme_signers_add (pop->gctx, seahorse_gpgme_key_get_private (SEAHORSE_GPGME_KEY (signkey)));
         gerr = gpgme_op_encrypt_sign_start (pop->gctx, recips, GPGME_ENCRYPT_ALWAYS_TRUST, 
                                             plain, cipher);
     } else {
@@ -309,7 +384,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
                                        plain, cipher);
     }
     
-    gpgmex_free_keys (recips);
+    free_keys (recips);
 
     /* Frees cipher */
     ret = process_crypto_result (pop, gerr, cipher, crypttext, error);
@@ -319,6 +394,21 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     return ret;
 }
 
+/**
+ * seahorse_service_crypto_sign_text:
+ * @crypto: SeahorseServiceCrypto
+ * @signer: the signer keyid
+ * @flags: 0 (ignored)
+ * @cleartext: the text to sign
+ * @crypttext: the clear text signature (out) (GPGME_SIG_MODE_CLEAR)
+ * @error: an error to return
+ *
+ * DBus: SignText
+ *
+ * Signs the @cleartext and returns the signature in @crypttext
+ *
+ * Returns: TRUE on success
+ */
 gboolean
 seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *signer, 
                                    int flags, const char *cleartext, char **crypttext,
@@ -326,7 +416,7 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
 {
     SeahorseObject *signkey = NULL;
     gpgme_error_t gerr;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     gpgme_data_t plain, cipher;
     gboolean ret = TRUE;
     
@@ -340,21 +430,21 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("No signer specified"));
     
-    signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer, NULL);
+    signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer);
     if (!signkey) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
                      _("Invalid or unrecognized signer: %s"), signer);
         return FALSE;
     }
         
-    if (!SEAHORSE_IS_PGP_KEY (signkey) || 
-        !(seahorse_object_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
+    if (!SEAHORSE_IS_GPGME_KEY (signkey) || 
+        !(seahorse_object_get_flags (signkey) & SEAHORSE_FLAG_CAN_SIGN)) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("Key is not valid for signing: %s"), signer);
         return FALSE;
     }
     
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data from text */
     gerr = gpgme_data_new_from_mem (&plain, cleartext, strlen (cleartext), FALSE);
@@ -367,7 +457,7 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
     gpgme_set_armor (pop->gctx, TRUE);
 
     /* Do the signage */
-    gpgme_signers_add (pop->gctx, SEAHORSE_PGP_KEY (signkey)->seckey);
+    gpgme_signers_add (pop->gctx, seahorse_gpgme_key_get_private (SEAHORSE_GPGME_KEY (signkey)));
     gerr = gpgme_op_sign_start (pop->gctx, plain, cipher, GPGME_SIG_MODE_CLEAR);
 
     /* Frees cipher */
@@ -378,6 +468,23 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
     return ret;
 }
 
+/**
+ * seahorse_service_crypto_decrypt_text:
+ * @crypto: #SeahorseServiceCrypto context
+ * @ktype: "openpgp"
+ * @flags: FLAG_QUIET for no notification
+ * @crypttext: the text to decrypt
+ * @cleartext: the decrypted text (out)
+ * @signer: the signer if the text is signed (out)
+ * @error: a potential error (out)
+ *
+ * DBus: DecryptText
+ *
+ * Decrypts the @crypttext and returns it in @cleartext. If the text
+ * was signed, the signed is returned.
+ *
+ * Returns: TRUE on success
+ */
 gboolean
 seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto, 
                                       const char *ktype, int flags, 
@@ -386,7 +493,7 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
 {
     gpgme_verify_result_t status;
     gpgme_error_t gerr;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     gpgme_data_t plain, cipher;
     gboolean ret = TRUE;
     GQuark keyid;
@@ -402,7 +509,7 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
      * then all this logic will need to change. 
      */
     
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data from text */
     gerr = gpgme_data_new_from_mem (&cipher, crypttext, strlen (crypttext), FALSE);
@@ -430,8 +537,8 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
             if (status->signatures->summary & GPGME_SIGSUM_GREEN ||
                 status->signatures->summary & GPGME_SIGSUM_VALID ||
                 status->signatures->summary & GPGME_SIGSUM_KEY_MISSING) {
-                keyid = seahorse_pgp_key_get_cannonical_id (status->signatures->fpr);
-                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid, 0);
+                keyid = seahorse_pgp_key_canonize_id (status->signatures->fpr);
+                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid);
             }
         }
     }
@@ -441,6 +548,23 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
     return ret;
 }
 
+/**
+ * seahorse_service_crypto_verify_text:
+ * @crypto: #SeahorseServiceCrypto context
+ * @ktype: "openpgp"
+ * @flags: FLAG_QUIET for no notification
+ * @crypttext: the text to decrypt
+ * @cleartext: the plaintext after verification (out)
+ * @signer: the signer if the text is signed (out)
+ * @error: a potential error (out)
+ *
+ * DBus: VerifyText
+ *
+ * Decrypts the @crypttext and returns it in @cleartext. If the text
+ * was signed, the signed is returned.
+ *
+ * Returns: TRUE on success
+ */
 gboolean
 seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto, 
                                      const gchar *ktype, int flags, 
@@ -449,7 +573,7 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
 {
     gpgme_verify_result_t status;
     gpgme_error_t gerr;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     gpgme_data_t plain, cipher;
     gboolean ret = TRUE;
     GQuark keyid;
@@ -465,7 +589,7 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
      * then all this logic will need to change. 
      */
 
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data from text */
     gerr = gpgme_data_new_from_mem (&cipher, crypttext, strlen (crypttext), FALSE);
@@ -493,8 +617,8 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
             if (status->signatures->summary & GPGME_SIGSUM_GREEN ||
                 status->signatures->summary & GPGME_SIGSUM_VALID ||
                 status->signatures->summary & GPGME_SIGSUM_KEY_MISSING) {
-                keyid = seahorse_pgp_key_get_cannonical_id (status->signatures->fpr);
-                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid, 0);
+                keyid = seahorse_pgp_key_canonize_id (status->signatures->fpr);
+                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid);
             }
         }
     }
