@@ -409,8 +409,11 @@ parse_hkp_index (const gchar *response)
     
 	for (l = lines; *l; l++) {
 
-		line = *l;
+		line = *l;	
 		dehtmlize (line);
+#if DEBUG_HKP_ENABLE
+        fprintf(stderr,"%s\n", line);
+#endif
 
 		/* Start a new key */
 		if (g_ascii_strncasecmp (line, "pub ", 4) == 0) {
@@ -519,9 +522,9 @@ parse_hkp_index (const gchar *response)
 	g_strfreev (lines);
 
 	if (key) {
-		seahorse_pgp_key_set_uids (SEAHORSE_PGP_KEY (key), uids);
+		seahorse_pgp_key_set_uids (SEAHORSE_PGP_KEY (key), g_list_reverse (uids));
 		seahorse_object_list_free (uids);
-		seahorse_pgp_key_set_subkeys (SEAHORSE_PGP_KEY (key), subkeys);
+		seahorse_pgp_key_set_subkeys (SEAHORSE_PGP_KEY (key), g_list_reverse (subkeys));
 		seahorse_object_list_free (subkeys);
 	}
 	
@@ -777,7 +780,7 @@ is_hex_keyid (const gchar *match)
     if (strlen (match) != 8)
         return FALSE;
     
-    while (*t != NULL) {
+    while (*t != '\0') {
         if (!((*t >= 0x30 && *t <= 0x39) || (*t >= 0x41 && *t <= 0x46) || (*t >= 0x61 && *t <= 0x66)))
             return FALSE;
             
@@ -1059,7 +1062,7 @@ seahorse_hkp_is_valid_uri (const gchar *uri)
         /* Must be http or https, have a host. No querystring, user, path, passwd etc... */
         if ((soup->scheme == SOUP_URI_SCHEME_HTTP || soup->scheme == SOUP_URI_SCHEME_HTTPS) && 
             !soup->user && !soup->password && !soup->query && !soup->fragment &&
-            g_str_equal (soup->path, "/"))
+            soup->host && g_str_equal (soup->path ? soup->path : "/", "/"))
             ret = TRUE;
         soup_uri_free (soup);
     }
